@@ -15,10 +15,10 @@ Built-in format strings
 module Time.Format
   ( -- * Parsing and Printing
     -- ** Format strings
-    TimeFormatElem (..)
-  , TimeFormatFct (..)
+    TimeFormat (..)
   , TimeFormatString (..)
-  , TimeFormat (..)
+  , TimeFormatElem (..)
+  , TimeFormatFct (..)
     -- ** Common built-in formats
   , ISO8601_Date (..)
   , ISO8601_DateAndTime (..)
@@ -31,13 +31,18 @@ module Time.Format
   , localTimeParseE
   ) where
 
-import           Data.Char ( isDigit, ord )
+import           Data.Char ( isDigit, isSpace, ord )
 import           Data.Int ( Int64 )
+import           Time.Calendar ( getDayOfTheYear )
 import           Time.Internal ( dateTimeFromUnixEpochP )
 import           Time.LocalTime
                    ( LocalTime (..), localTime, localTimeToGlobal )
-import           Time.Time ( Timeable (..), timeGetDateTimeOfDay )
-import           Time.Utils ( pad2, pad4, padN )
+import           Time.Time
+                   ( Period (..), Timeable (..), dateAddPeriod
+                   , timeGetDateTimeOfDay
+                   )
+import           Time.Timezone ( Timezone (..), TimezoneMinutes (..) )
+import           Time.Utils ( pad2, pad3, pad4, padN )
 import           Time.Types
                    ( Date (..), DateTime (..), Elapsed (..), ElapsedP (..)
                    , Hours (..), Minutes (..), Month (..), NanoSeconds (..)
@@ -48,71 +53,90 @@ import           Time.Types
 -- | Type representing formatters that can be part of a time format string.
 data TimeFormatElem =
     Format_Year2
-    -- ^ 2 digit years (70 is 1970, 69 is 2069).
+    -- ^ 2-digit years (@70@ is 1970, @69@ is 2069).
   | Format_Year4
-    -- ^ 4 digits years.
+    -- ^ 4-digit years.
   | Format_Year
     -- ^ Any digits years.
   | Format_Month
-    -- ^ Months (1 to 12).
+    -- ^ Months (@1@ to @12@).
   | Format_Month2
-    -- ^ Months padded to 2 characters (01 to 12).
+    -- ^ Months padded to 2 characters (@01@ to @12@).
   | Format_MonthName_Short
-    -- ^ Short name of nthe month (\'Jan\', \'Feb\' ..).
+    -- ^ Short name of the month (@Jan@, @Feb@, ..).
   | Format_DayYear
-    -- ^ Day of the year (1 to 365, 366 for leap years).
+    -- ^ Day of the year (@1@ to @365@, @366@ for leap years).
+  | Format_DayYear3
+    -- ^ Day of the year padded to 3 characters (@001@ to @365@, @366@ for leap
+    -- years).
   | Format_Day
-    -- ^ Day of the month (1 to 31).
+    -- ^ Day of the month (@1@ to @31@).
   | Format_Day2
-    -- ^ Day of the month padded to 2 characters (01 to 31).
+    -- ^ Day of the month padded to 2 characters (@01@ to @31@).
   | Format_Hour
-    -- ^ Hours (0 to 23).
+    -- ^ Hours padded to 2 characters (@00@ to @23@).
   | Format_Minute
-    -- ^ Minutes (0 to 59).
+    -- ^ Minutes padded to 2 characters (@00@ to @59@).
   | Format_Second
-    -- ^ sSeconds (0 to 59, 60 for leap seconds).
+    -- ^ Seconds padded to 2 characters (@00@ to @59@, @60@ for leap seconds).
   | Format_UnixSecond
-    -- ^ Number of seconds since the start of the Unix epoch
+    -- ^ Number of non-leap seconds since the Unix epoch
     -- (1970-01-01 00:00:00 UTC).
   | Format_MilliSecond
-    -- ^ Milliseconds padded to 3 characters (000 to 999).
+    -- ^ The millisecond component only, padded to 3 characters (@000@ to
+    -- @999@). See 'Format_MicroSecond' and 'Format_NanoSecond' for other
+    -- named sub-second components.
   | Format_MicroSecond
-    -- ^ MicroSeconds padded to 6 characters (000000 to 999999).
+    -- ^ The microseconds component only, padded to 3 characters (@000@ to
+    -- @999@). See 'Format_MilliSecond' and 'Format_NanoSecond' for other
+    -- named sub-second components.
   | Format_NanoSecond
-    -- ^ NanoSeconds padded to 9 characters (000000000 to 999999999).
+    -- ^ The nanoseconds component only, padded to 3 characters (@000@ to
+    -- @999@). See 'Format_MilliSecond' and 'Format_MicroSecond' for other
+    -- named sub-second components.
   | Format_Precision Int
-    -- ^ Sub seconds display with a precision of n digits, with n between @1@
+    -- ^ Sub-second display with a precision of n digits, with n between @1@
     -- and @9@.
   | Format_TimezoneName
-    -- ^ Timezone name (e.g. GMT, PST). Not yet implemented.
---  Format_TimezoneOffset
-    -- ^ Timezone offset offset (+02:00).
+    -- ^ Timezone name.
   | Format_TzHM_Colon_Z
-    -- ^ Zero UTC offset (Z) or timezone offset with colon (+02:00).
+    -- ^ Zero UTC offset (@Z@) or timezone offset with colon (for example,
+    -- @02:00@, @+02:00@ or @-02:00@).
   | Format_TzHM_Colon
-    -- ^ Timezone offset with colon (+02:00).
+    -- ^ Timezone offset with colon (for example, @02:00@, @+02:00@ or
+    -- @-02:00@).
   | Format_TzHM
-    -- ^ Timezone offset without colon (+0200).
+    -- ^ Timezone offset without colon (for example, @0200@, @+0200@ or
+    -- @-0200@).
   | Format_Tz_Offset
-    -- ^ Timezone offset in minutes.
+    -- ^ Timezone offset in minutes (for example, @120@, @+120@ or @-120@).
   | Format_Spaces
-    -- ^ One or more space-like chars.
+    -- ^ One or more space-like characters.
   | Format_Text Char
     -- ^ A verbatim character.
   | Format_Fct TimeFormatFct
-    -- ^ Not implemented.
+    -- ^ A custom time format function. See v'TimeFormatFct'.
   deriving (Eq, Show)
 
 -- | Type representing format functions.
 data TimeFormatFct = TimeFormatFct
   { timeFormatFctName :: String
     -- ^ The name of the format function.
-  , timeFormatParse   :: DateTime -> String -> Either String (DateTime, String)
-    -- ^ A parser.
-  , timeFormatPrint   :: DateTime -> String
-    -- A printer.
+  , timeFormatParse ::
+         (DateTime, TimezoneOffset)
+      -> String
+      -> Either String ((DateTime, TimezoneOffset), String)
+    -- ^ A parser of a given 'String'. The first argument is the
+    -- @(@v'DateTime'@,@ v'TimezoneOffset'@)@ value before the parser is
+    -- applied. If the parser fails, the 'Left' value provides an error message.
+    -- If it succeeds, the 'Right' value provides a pair of a
+    -- @(@v'DateTime'@,@ v'TimezoneOffset'@)@ value and any input not consumed
+    -- by the parser.
+  , timeFormatPrint :: DateTime -> TimezoneOffset -> String
+    -- ^ A printer of a given v'DateTime' value and v'TimezoneOffset' value.
   }
 
+-- | Show the 'timeFormatFctName' field.
 instance Show TimeFormatFct where
   show = timeFormatFctName
 
@@ -126,6 +150,57 @@ newtype TimeFormatString = TimeFormatString [TimeFormatElem]
 
 -- | A type class promising the ability to convert values to
 -- a t'TimeFormatString'.
+--
+-- 'String' is an instance of 'TimeFormat'. Sequences of characters are
+-- interpreted as follows (case-sensitive). The longest valid sequence is parsed
+-- first:
+--
+-- [@YY@]:    'Format_Year2'. 2-digit years (@70@ is 1970, @69@ is 2069).
+-- [@YYYY@]:  'Format_Year4'. 4-digit years.
+-- [@M@]:     'Format_Month'. Months (@1@ to @12@).
+-- [@MM@]:    'Format_Month2'. Months padded to 2 characters (@01@ to @12@).
+-- [@Mon@]:   'Format_MonthName_Short'. Short name of the month (@Jan@, @Feb@,
+--            ..).
+-- [@JJJ@]:   'Format_DayYear3'. Day of the year padded to 3 characters (@001@
+--            to @365@, @366@ for leap years).
+-- [@DD@]:    'Format_Day2'. Day of the month padded to 2 characters (@01@ to
+--            @31@).
+-- [@H@]:     'Format_Hour'. Hours padded to 2 characters (@00@ to @23@).
+-- [@MI@]:    'Format_Minute'. Minutes padded to 2 characters (@00@ to @59@).
+-- [@S@]:     'Format_Second'. Seconds padded to 2 characters (@00@ to @59@,
+--            @60@ for leap seconds).
+-- [@EPOCH@]: 'Format_UnixSecond'. Number of non-leap seconds since the Unix
+--            epoch (1970-01-01 00:00:00 UTC).
+-- [@ms@]:    'Format_MilliSecond'. The millisecond component only, padded to 3
+--            characters (@000@ to @999@). See @us@/@μ@ and @ns@ for other named
+--            sub-second components.
+-- [@us@]:    'Format_MicroSecond'. The microseconds component only, padded to 3
+--            characters (@000@ to @999@). See @ms@ and @ns@ for other named
+--            sub-second components.
+-- [@μ@]:     'Format_MicroSecond'. As above.
+-- [@ns@]:    'Format_NanoSecond'. The nanoseconds component only, padded to 3
+--            characters (@000@ to @999@). See @ms@ and @us@/@μ@ for other named
+--            sub-second components.
+-- [@p\<n\>@]: 'Format_Precision' @\<n\>@. Sub-second display with a precision
+--             of @\<n\>@ digit(s), where @\<n\>@ is @1@ to @9@.
+-- [@TZH:M@]: 'Format_TzHM_Colon'. Timezone offset with colon (for example,
+--            @02:00@, @+02:00@ or @-02:00@).
+-- [@TZHM@]:  'Format_TzHM'. Timezone offset without colon (for example,
+--            @0200@, @+0200@ or @-0200@).
+-- [@TZOFS@]: 'Format_Tz_Offset'. Timezone offset in minutes (for example,
+--            @120@, @+120@ or @-120@).
+-- [@\<space\>@]:       'Format_Spaces'. One or more space-like characters.
+-- [@\\\<character\>@]: 'Format_Text' @\<character\>@. A verbatim character.
+-- [@\<character\>@]:   'Format_Text' @\<character\>@. A verbatim character.
+--
+-- For example:
+--
+-- >>> let mDateTime = timeParse ("ms \\ms us \\us ns \\ns") "123 ms 456 us 789 ns"
+-- >>> timeGetNanoSeconds <$> mDateTime
+-- Just 123456789ns
+--
+-- >>> timePrint "ms \\ms us \\us ns \\ns" <$> mDateTime
+-- Just "123 ms 456 us 789 ns"
 class TimeFormat format where
   toFormat :: format -> TimeFormatString
 
@@ -147,6 +222,8 @@ instance TimeFormat [TimeFormatElem] where
 instance TimeFormat TimeFormatString where
   toFormat = id
 
+-- | For information about this instance, see the documentation for
+-- 'TimeFormat'.
 instance TimeFormat String where
   toFormat = TimeFormatString . toFormatElem
    where
@@ -157,6 +234,7 @@ instance TimeFormat String where
     toFormatElem ('M':'o':'n':r)     = Format_MonthName_Short : toFormatElem r
     toFormatElem ('M':'I':r)         = Format_Minute : toFormatElem r
     toFormatElem ('M':r)             = Format_Month  : toFormatElem r
+    toFormatElem ('J':'J':'J':r)     = Format_DayYear3 : toFormatElem r
     toFormatElem ('D':'D':r)         = Format_Day2   : toFormatElem r
     toFormatElem ('H':r)             = Format_Hour   : toFormatElem r
     toFormatElem ('S':r)             = Format_Second : toFormatElem r
@@ -186,16 +264,17 @@ instance TimeFormat String where
 
 instance TimeFormat ISO8601_Date where
   toFormat _ =
-    TimeFormatString [Format_Year,dash,Format_Month2,dash,Format_Day2]
+    TimeFormatString [Format_Year, dash, Format_Month2, dash, Format_Day2]
    where
     dash = Format_Text '-'
 
 instance TimeFormat ISO8601_DateAndTime where
   toFormat _ = TimeFormatString
-    [ Format_Year,dash,Format_Month2,dash,Format_Day2 -- date
+    [ Format_Year, dash, Format_Month2, dash, Format_Day2 -- date
     , Format_Text 'T'
-    , Format_Hour,colon,Format_Minute,colon,Format_Second -- time
-    , Format_TzHM_Colon_Z -- zero UTC offset (Z) or timezone offset with colon +HH:MM
+    , Format_Hour, colon, Format_Minute, colon, Format_Second -- time
+    , Format_TzHM_Colon_Z
+      -- Either timezone offset with colon (±HH:MM) or UTC zero offset (Z).
     ]
    where
     dash = Format_Text '-'
@@ -221,10 +300,10 @@ monthFromShort str =
 printWith ::
      (TimeFormat format, Timeable t)
   => format
-  -> TimezoneOffset
   -> t
+  -> TimezoneOffset
   -> String
-printWith fmt tzOfs@(TimezoneOffset tz) t = concatMap fmtToString fmtElems
+printWith fmt t tzOfs@(TimezoneOffset tz) = concatMap fmtToString fmtElems
  where
   fmtToString Format_Year     = show (dateYear date)
   fmtToString Format_Year4    = pad4 (dateYear date)
@@ -232,6 +311,8 @@ printWith fmt tzOfs@(TimezoneOffset tz) t = concatMap fmtToString fmtElems
   fmtToString Format_Month2   = pad2 (fromEnum (dateMonth date) + 1)
   fmtToString Format_Month    = show (fromEnum (dateMonth date) + 1)
   fmtToString Format_MonthName_Short = take 3 $ show (dateMonth date)
+  fmtToString Format_DayYear  = show (getDayOfTheYear date)
+  fmtToString Format_DayYear3 = pad3 (getDayOfTheYear date)
   fmtToString Format_Day2     = pad2 (dateDay date)
   fmtToString Format_Day      = show (dateDay date)
   fmtToString Format_Hour     = pad2 (fromIntegral (todHour tm) :: Int)
@@ -244,7 +325,7 @@ printWith fmt tzOfs@(TimezoneOffset tz) t = concatMap fmtToString fmtElems
       | n >= 1 && n <= 9 = padN n (ns `div` (10 ^ (9 - n)))
       | otherwise        = error "invalid precision format"
   fmtToString Format_UnixSecond = show unixSecs
-  fmtToString Format_TimezoneName   = "" --
+  fmtToString Format_TimezoneName = timezoneName $ TimezoneMinutes tz
   fmtToString Format_Tz_Offset = show tz
   fmtToString Format_TzHM = show tzOfs
   fmtToString Format_TzHM_Colon_Z
@@ -256,36 +337,92 @@ printWith fmt tzOfs@(TimezoneOffset tz) t = concatMap fmtToString fmtElems
        in sign ++ pad2 tzH ++ ":" ++ pad2 tzM
   fmtToString Format_Spaces   = " "
   fmtToString (Format_Text c) = [c]
-  fmtToString f = error ("unimplemented printing format: " ++ show f)
+  fmtToString (Format_Fct tff) = timeFormatPrint tff dateTime tzOfs
 
   (TimeFormatString fmtElems) = toFormat fmt
 
   (Elapsed (Seconds unixSecs)) = timeGetElapsed t
-  (DateTime date tm) = timeGetDateTimeOfDay t
+  dateTime@(DateTime date tm) = timeGetDateTimeOfDay t
   (NanoSeconds ns) = timeGetNanoSeconds t
 
 -- | Given the specified format, pretty print the given local time.
+--
+-- A v'Format_TimezoneName' will print using the ±HH:MM format, where @0@
+-- offset is printed as @-00:00@.
+--
+-- A v'Format_TzHM' will print using the ±HHMM format, where @0@ offset is
+-- printed as @+0000@.
+--
+-- A v'Format_TzHM_Colon' will print using the ±HH:MM format, where @0@ offset
+-- is printed as @+00:00@.
+--
+-- A v'Format_TzHM_Colon_Z' will print using the ±HH:MM format, but where @0@
+-- offset is printed as @Z@.
+--
+-- A v'Format_Tz_Offset' will print non-negative offsets without using an
+-- initial @+@.
+--
+-- A v'Format_Spaces' will print a single space character.
 localTimePrint ::
      (TimeFormat format, Timeable t)
   => format      -- ^ The format to use for printing.
   -> LocalTime t -- ^ The local time to print.
   -> String
 localTimePrint fmt lt =
-  localTimeUnwrap $ fmap (printWith fmt (localTimeGetTimezone lt)) lt
+  printWith fmt (localTimeUnwrap lt) (localTimeGetTimezone lt)
 
--- | Given the specified format, pretty print the given time.
+-- | Like 'localTimePrint' but the time zone of the time to print will be taken
+-- to be UTC.
 timePrint ::
      (TimeFormat format, Timeable t)
   => format -- ^ The format to use for printing.
   -> t      -- ^ The time to print.
   -> String
-timePrint fmt = printWith fmt timezone_UTC
+timePrint fmt t = printWith fmt t timezone_UTC
 
--- | Given the specified format, try to parse the given string as time value.
+-- | Given the specified format, try to parse the given string as
+-- a t'LocalTime' t'DateTime' value.
 --
--- On failure, the parsing function returns the reason of the failure.
+-- On failure, yields a 'Left' value with a pair of the
+-- current t'TimeFormatElem' value and the reason for the failure.
 --
--- If successful, yield the parsed value and the remaining unparsed string.
+-- If successful, yields a 'Right' value with a pair of the parsed value and
+-- the remaining unparsed string.
+--
+-- The default parsed t'LocalTime' t'DateTime' value is \'all zeros'\. For
+-- example:
+--
+-- >>> let zeroDate = Date 0 January 0
+-- >>> let zeroTime = TimeOfDay 0 0 0 0
+-- >>> let zeroLocalTime = localTime timezone_UTC (DateTime zeroDate zeroTime)
+-- >>> localTimeParseE "" "" == Right (zeroLocalTime, "")
+-- True
+--
+-- Later t'TimeFormatElem' values can modify the result of earlier
+-- t'TimeFormatElem' values. For example:
+--
+-- >>> let toYear = dateYear . dtDate . localTimeUnwrap . fst
+-- >>> toYear <$> (localTimeParseE "YYYY YYYY" "2025 2024")
+-- Right 2024
+--
+-- A v'Format_DayYear' or v'Format_DayYear3' value interprets the day of year
+-- based on the previously parsed year or, by default, a leap year. For example:
+--
+-- >>> let toMonth = dateMonth . dtDate . localTimeUnwrap . fst
+-- >>> let format1 = [Format_Year4, Format_Spaces, Format_DayYear]
+-- >>> let format2 = [Format_DayYear, Format_Spaces, Format_Year4]
+-- >>> toMonth <$> (localTimeParseE format1 "2025 60")
+-- Right March
+-- >>> toMonth <$> (localTimeParseE format2 "60 2025")
+-- Right February
+--
+-- A v'Format_TimezoneName' will parse one or more non-white space characters
+-- but will not modify the previously parsed, or default, date and time.
+--
+-- A v'Format_Month', v'Format_DayYear', v'Format_DayYear3', v'Format_Day'
+-- and v'Format_Tz_Offset' will check that the parsed number is within bounds.
+-- However, 'localTimeParseE' does not check that any resulting date or time is
+-- a valid one.
 localTimeParseE ::
      TimeFormat format
   => format -- ^ The format to use for parsing.
@@ -304,10 +441,15 @@ localTimeParseE fmt = loop ini fmtElems
       Left err         -> Left (x, err)
       Right (nacc, s') -> loop nacc xs s'
 
+  processOne ::
+       (DateTime, TimezoneOffset)
+    -> TimeFormatElem
+    -> [Char]
+    -> Either String ((DateTime, TimezoneOffset), [Char])
   processOne _   _               []     = Left "empty"
   processOne acc (Format_Text c) (x:xs)
     | c == x    = Right (acc, xs)
-    | otherwise = Left ("unexpected char, got: " ++ show c)
+    | otherwise = Left ("unexpected char, got: " ++ show x)
 
   processOne acc Format_Year s =
     onSuccess (\y -> modDate (setYear y) acc) $ isNumber s
@@ -318,12 +460,55 @@ localTimeParseE fmt = loop ini fmtElems
             in  modDate (setYear year) acc
     )
     $ getNDigitNum 2 s
+  processOne acc Format_Month s =
+    let result = isNumber s :: Either String (Int, String)
+    in  case result of
+          Left err -> Left err
+          Right (m, s')
+            | m > 0 && m <= 12 ->
+                Right (modDate (setMonth (toEnum (m - 1))) acc, s')
+            | otherwise -> Left ("month invalid, got: " <> show m)
   processOne acc Format_Month2 s = onSuccess
     ( \m -> modDate (setMonth $ toEnum ((fromIntegral m - 1) `mod` 12)) acc
     )
     $ getNDigitNum 2 s
   processOne acc Format_MonthName_Short s =
     onSuccess (\m -> modDate (setMonth m) acc) $ getMonth s
+  processOne acc Format_DayYear s =
+    let y = (dateYear . dtDate . fst) acc
+        result = isNumber s :: Either String (Int, String)
+    in  case result of
+          Left err -> Left err
+          Right (d, s')
+              -- We can't be more helpful because we may not yet know the
+              -- intended year
+            | d > 0 && d <= 366 ->
+                let p = Period 0 0 (d - 1)
+                    startOfYear = Date y January 1
+                in  Right (modDate (const (dateAddPeriod startOfYear p)) acc, s')
+            | otherwise -> Left ("day of year invalid, got: " <> show d)
+  processOne acc Format_DayYear3 s =
+    let y = (dateYear . dtDate . fst) acc
+        result = getNDigitNum 3 s :: Either String (Int64, String)
+    in  case result of
+          Left err -> Left err
+          Right (d, s')
+              -- We can't be more helpful because we may not yet know the
+              -- intended year
+            | d > 0 && d <= 366 ->
+                let p = Period 0 0 (fromIntegral d - 1)
+                    startOfYear = Date y January 1
+                in  Right (modDate (const (dateAddPeriod startOfYear p)) acc, s')
+            | otherwise -> Left ("day of year invalid, got: " <> show d)
+  processOne acc Format_Day s =
+    let result = isNumber s :: Either String (Int, String)
+    in  case result of
+          Left err -> Left err
+          Right (d, s')
+              -- We can't be more helpful because we may not yet know the
+              -- intended month and year
+            | d > 0 && d <= 31 -> Right (modDate (setDay d) acc, s')
+            | otherwise -> Left ("day of month invalid, got: " <> show d)
   processOne acc Format_Day2 s =
     onSuccess (\d -> modDate (setDay d) acc) $ getNDigitNum 2 s
   processOne acc Format_Hour s =
@@ -340,6 +525,9 @@ localTimeParseE fmt = loop ini fmtElems
     onSuccess (\ns -> modTime (setNsMask (0,3) ns) acc) $ getNDigitNum 3 s
   processOne acc (Format_Precision p) s =
     onSuccess (\num -> modTime (setNS num) acc) $ getNDigitNum p s
+  processOne acc Format_TimezoneName s = case break isSpace s of
+    ("", _) -> Left ("no non-white space at start of: " <> s)
+    (_, s2) -> Right (acc, s2)
   processOne acc Format_UnixSecond s =
     onSuccess (\sec ->
       let newDate =
@@ -352,10 +540,27 @@ localTimeParseE fmt = loop ini fmtElems
     parseHMSign True acc c s
   processOne acc Format_TzHM (c:s) =
     parseHMSign False acc c s
-
-  processOne acc Format_Spaces (' ':s) = Right (acc, s)
-  -- catch all for unimplemented format.
-  processOne _ f _ = error ("unimplemented parsing format: " ++ show f)
+  processOne acc Format_Tz_Offset s@(c:cs) = case c of
+    '-' -> process True (12 * 60) cs
+    '+' -> process False (14 * 60) cs
+    _ -> process False (14 * 60) s
+   where
+    (dt, _) = acc
+    process isNeg limit s' =
+      let result = isNumber s' :: Either String (Int, String)
+      in  case result of
+            Left err -> Left err
+            Right (mins, rest)
+              | mins >= 0 && mins <= limit ->
+                  let mins' = if isNeg then negate mins else mins
+                  in Right ((dt, TimezoneOffset mins'), rest)
+              | otherwise ->
+                let sign = if isNeg then "-" else "+"
+                in Left ("offset invalid, got: " <> sign <> show mins)
+  processOne acc Format_Spaces s = case span isSpace s of
+    ("", _) -> Left ("no white space at start of: " <> s)
+    (_, s2) -> Right (acc, s2)
+  processOne acc (Format_Fct tff) s = timeFormatParse tff acc s
 
   parseHMSign expectColon acc signChar afterSign =
     case signChar of
@@ -366,11 +571,11 @@ localTimeParseE fmt = loop ini fmtElems
   parseHM isNeg True (h1:h2:':':m1:m2:xs) acc
     | allDigits [h1,h2,m1,m2] = let tz = toTZ isNeg h1 h2 m1 m2
                                 in  Right (modTZ (const tz) acc, xs)
-    | otherwise = Left ("not digits chars: " ++ show [h1,h2,m1,m2])
+    | otherwise = Left ("non-digit char(s) in: " ++ show [h1,h2,m1,m2])
   parseHM isNeg False (h1:h2:m1:m2:xs) acc
     | allDigits [h1,h2,m1,m2] = let tz = toTZ isNeg h1 h2 m1 m2
                                 in  Right (modTZ (const tz) acc, xs)
-    | otherwise = Left ("not digits chars: " ++ show [h1,h2,m1,m2])
+    | otherwise = Left ("non-digit char(s) in: " ++ show [h1,h2,m1,m2])
   parseHM _ _    _ _ = Left "invalid timezone format"
 
   toTZ isNeg h1 h2 m1 m2 = TimezoneOffset ((if isNeg then negate else id) minutes)
@@ -383,7 +588,7 @@ localTimeParseE fmt = loop ini fmtElems
   isNumber :: Num a => String -> Either String (a, String)
   isNumber s =
     case span isDigit s of
-      ("", s2) -> Left ("no digits chars:" ++ s2)
+      ("", s2) -> Left ("no digits at start of:" ++ s2)
       (s1, s2) -> Right (toInt s1, s2)
 
   getNDigitNum :: Int -> String -> Either String (Int64, String)
@@ -391,7 +596,7 @@ localTimeParseE fmt = loop ini fmtElems
     case getNChar n s of
       Left err                            -> Left err
       Right (s1, s2)
-        | not (allDigits s1) -> Left ("not a digit chars in " ++ show s1)
+        | not (allDigits s1) -> Left ("non-digit char(s) in " ++ show s1)
         | otherwise          -> Right (toInt s1, s2)
 
   getMonth :: String -> Either String (Month, String)
@@ -437,13 +642,10 @@ localTimeParseE fmt = loop ini fmtElems
     s = 10 ^ shift
     m = 10 ^ mask
 
--- | Given the specified format, try to parse the given string as time value.
+-- | Like 'localTimeParseE', but with simpler handing of failure. Does not yield
+-- the remaining unparsed string on success.
 --
--- On failure, returns 'Nothing'.
---
--- If successful, yields 'Just' the parsed value.
---
--- For more elaborate needs use 'localTimeParseE'.
+-- On failure, returns 'Nothing'. If successful, yields 'Just' the parsed value.
 localTimeParse ::
      TimeFormat format
   => format -- ^ The format to use for parsing.
