@@ -1,6 +1,7 @@
 {-# OPTIONS_GHC -Wno-orphans            #-}
 
 {-# LANGUAGE FlexibleInstances   #-}
+{-# LANGUAGE NumericUnderscores  #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
 module Main
@@ -9,16 +10,18 @@ module Main
 
 import qualified Control.Exception as E
 import           Control.Monad ( when )
+import           CustomTimeFormatFct ( customTimeFormatFct )
 import           Data.Hourglass
                    ( Date (..), DateTime (..), Duration (..), Elapsed (..)
                    , ElapsedP (..), Hours (..), ISO8601_Date (..)
                    , ISO8601_DateAndTime (..), LocalTime, Minutes (..)
                    , Month (..), NanoSeconds (..), Period (..), Seconds (..)
-                   , Time, TimeFormat (..), TimeOfDay (..), TimezoneOffset (..)
-                   , WeekDay, dateAddPeriod, daysInMonth, getWeekDay, localTime
-                   , localTimeFromGlobal, localTimeGetTimezone, localTimeParseE
-                   , localTimeSetTimezone, localTimeToGlobal, timeConvert
-                   , timeGetDateTimeOfDay, timeGetElapsed, timeParseE, timePrint
+                   , Time, TimeFormat (..), TimeFormatElem (..), TimeOfDay (..)
+                   , TimezoneOffset (..), WeekDay, dateAddPeriod, daysInMonth, fromRationalElapsedP
+                   , getWeekDay, localTime, localTimeFromGlobal
+                   , localTimeGetTimezone, localTimeParseE, localTimeSetTimezone
+                   , localTimeToGlobal, timeConvert, timeGetDateTimeOfDay
+                   , timeGetElapsed, timeParseE, timePrint
                    )
 import           Data.Int ( Int64 )
 import           Data.Ratio ( (%) )
@@ -31,7 +34,10 @@ import           Test.Tasty.HUnit
                    ( Assertion, (@=?), assertEqual, assertFailure, testCase )
 import           Test.Tasty.QuickCheck
                    ( Arbitrary (..), choose, elements, testProperty )
-import           Time.Epoch ( ElapsedSince, WindowsEpoch )
+import           Time.Epoch
+                   ( ElapsedSince (..), ElapsedSinceP (..), UnixEpoch
+                   , WindowsEpoch
+                   )
 import           TimeDB ( parseTimeConv )
 import           TimeRange ( dateRange, hiElapsed, loElapsed )
 
@@ -66,17 +72,20 @@ instance Arbitrary Seconds where
       | otherwise = error "internal error"
 
 instance Arbitrary Minutes where
-  arbitrary = Minutes <$> choose (-1125899906842624, 1125899906842624)
+  arbitrary = Minutes <$> choose (-1_125_899_906_842_624, 1_125_899_906_842_624)
 
 instance Arbitrary Hours where
-  arbitrary = Hours <$> choose (-1125899906842, 1125899906842)
+  arbitrary = Hours <$> choose (-1_125_899_906_842, 1_125_899_906_842)
 
 instance Arbitrary NanoSeconds where
-  arbitrary = NanoSeconds <$> choose (0, 100000000)
+  arbitrary = NanoSeconds <$> choose (0, 100_000_000)
 
 instance Arbitrary Elapsed where
   arbitrary = Elapsed <$> arbitrary
 
+instance Arbitrary ElapsedP where
+  arbitrary = ElapsedP <$> arbitrary <*> arbitrary
+
 instance Arbitrary TimezoneOffset where
   arbitrary = TimezoneOffset <$> choose (-(11*60), 11*60)
 
@@ -85,9 +94,9 @@ instance Arbitrary Duration where
 
 instance Arbitrary Period where
   arbitrary = Period
-    <$> choose (-29,29)
-    <*> choose (-27,27)
-    <*> choose (-400,400)
+    <$> choose (-29, 29)
+    <*> choose (-27, 27)
+    <*> choose (-400, 400)
 
 instance Arbitrary Month where
   arbitrary = elements [January ..]
@@ -103,8 +112,8 @@ instance Arbitrary Date where
 
 instance Arbitrary TimeOfDay where
   arbitrary = (TimeOfDay . Hours <$> choose (0, 23))
-    <*> (Minutes <$> choose (0,59))
-    <*> (Seconds <$> choose (0,59))
+    <*> (Minutes <$> choose (0, 59))
+    <*> (Seconds <$> choose (0, 59))
     <*> arbitrary
 
 instance (Time t, Arbitrary t) => Arbitrary (LocalTime t) where
@@ -133,9 +142,9 @@ testCaseWith what fun (x, y, ref) = testCase
 
 arithmeticTestAddRef :: [(ElapsedP, ElapsedP, ElapsedP)]
 arithmeticTestAddRef = map testRefToElapsedP
-  [ ((1, 090000000), (2, 090000000), (3, 180000000))
-  , ((1, 900000000), (1, 200000000), (3, 100000000))
-  , ((1, 000000001), (0, 999999999), (2, 000000000))
+  [ ((1, 090_000_000), (2, 090_000_000), (3, 180_000_000))
+  , ((1, 900_000_000), (1, 200_000_000), (3, 100_000_000))
+  , ((1, 000_000_001), (0, 999_999_999), (2, 000_000_000))
   ]
 
 arithmeticTestSubRef :: [(ElapsedP, ElapsedP, ElapsedP)]
@@ -146,7 +155,7 @@ arithmeticTestSubRef = map testRefToElapsedP
   , ((1, ms 100), (2, ms 400), (-2, ms 700))
   ]
  where
-  ms v = v * 1000000
+  ms v = v * 1_000_000
 
 testRefToElapsedP ::
      ((Int64, Int64), (Int64, Int64), (Int64, Int64))
@@ -174,6 +183,8 @@ tests knowns = testGroup "hourglass"
           let e2 = timeConvert e :: ElapsedSince WindowsEpoch
           in       timePrint ISO8601_DateAndTime e
               `eq` timePrint ISO8601_DateAndTime e2
+      , testProperty "precise seconds" $ \(ep :: ElapsedP) ->
+          fromRationalElapsedP (toRational ep) `eq` ep
       ]
   , testGroup "localtime"
       [ testProperty "eq" $ \(l :: LocalTime Elapsed) ->
@@ -253,16 +264,42 @@ tests knowns = testGroup "hourglass"
           test_property_format ("YYYY-MM-DDTH:MI:S.msusns" :: String)
       , testProperty "custom-2" $
           test_property_format ("Mon DD\\t\\h YYYY at HH\\hMI\\mS\\s.p9\\n\\s" :: String)
+      , testProperty "customTimeFormatFct" $
+          test_property_format
+            [ Format_Year4
+            , Format_Spaces
+            , Format_Fct customTimeFormatFct
+            , Format_Spaces
+            , Format_Hour
+            , Format_Spaces
+            , Format_Minute
+            , Format_Spaces
+            , Format_Second
+            , Format_Spaces
+            , Format_Precision 9
+            ]
       ]
   , testGroup "Regression Tests"
-      [ testCase  "Real instance of ElapsedP (#33)" $
+      [ testCase "Real instance of ElapsedP (#33)" $
           let res = toRational (ElapsedP (Elapsed $ Seconds 0) (NanoSeconds 0))
               ref = toRational (0 :: Int) :: Rational
           in  assertEqual "failed equality" ref res
-      , testCase  "Real instance of ElapsedP (#33) (2)" $
+      , testCase "Real instance of ElapsedP (#33) (2)" $
           let res = toRational
-                (ElapsedP (Elapsed $ Seconds 100) (NanoSeconds 1000000))
-              ref = toRational (100 :: Int) + (1 % 1000) :: Rational
+                (ElapsedP (Elapsed $ Seconds 100) (NanoSeconds 1_000_000))
+              ref = toRational (100 :: Int) + (1 % 1_000) :: Rational
+          in  assertEqual "failed equality" ref res
+      , testCase "Real instance of ElapsedSinceP, zero ns" $
+          let start = ElapsedSince (Seconds 0) :: ElapsedSince UnixEpoch
+              startP = ElapsedSinceP start (NanoSeconds 0) :: ElapsedSinceP UnixEpoch
+              res = toRational startP
+              ref = toRational (0 :: Int) :: Rational
+          in  assertEqual "failed equality" ref res
+      , testCase "Real instance of ElapsedSinceP, non zero ns" $
+          let later = ElapsedSince (Seconds 100) :: ElapsedSince UnixEpoch
+              laterP = ElapsedSinceP later (NanoSeconds 1_000_000) :: ElapsedSinceP UnixEpoch
+              res = toRational laterP
+              ref = toRational (100 :: Int) + (1 % 1_000) :: Rational
           in  assertEqual "failed equality" ref res
       ]
   ]
